<?php

/**
 * Special folders
 *
 * Plugin that adds folders to the special ones (shown at top of folders lists).
 *
 * @version 1.0
 */
class special_folders extends rcube_plugin
{
  public $task = 'mail|settings';
  private $special_folders_defaults = false;
  private $special_folders_user = false;
  private $special_folders = false;
  private $default_folders = false;
  private $rcmail;

  function init()
  {
    $rcmail = rcmail::get_instance();
    if ($rcmail->task == 'mail' && ($rcmail->action == '' || $rcmail->action == 'show'))
    {
      $this->_load_config();
      
      if ($this->special_folders || 1)
      {
        // Load plugin specific things
        $skin_path = $this->local_skin_path();
        $this->include_script($skin_path . '/special_folders.js');
        $this->add_texts('localization', true);
      
        // register hook to localize the special_folders folders
        $this->add_hook('render_mailboxlist', array($this, 'render_mailboxlist'));
        $this->default_folders = $rcmail->config->get('default_folders');

        // set env variable for client
        $special_folders_icons = array();
        foreach ($this->special_folders as $idx => $folder)
        {
          $special_folders_icons[$folder['folder']] = $this->url($skin_path . "/img/" . (strlen($folder['icon']) ? $folder['icon'] : 'foldericon.png'));
         
          // add special_folders folder to the list of default mailboxes
          if ($this->default_folders && !in_array($special_folders, $this->default_folders)) 
          {
            $this->default_folders[] = $folder['folder'];
          }  
        }
        
        // Set variables for JavaScript
        $rcmail->config->set('default_imap_folders', $this->default_folders);
        $rcmail->output->set_env('special_folders_icons', $special_folders_icons);
      }
    }
    else if ($rcmail->task == 'settings')
    {
      $dont_override = $rcmail->config->get('dont_override', array());
      if (!in_array('special_folders', $dont_override)) {
        $this->add_hook('user_preferences', array($this, 'prefs_table'));
        $this->add_hook('save_preferences', array($this, 'save_prefs'));
      }
    }
  }

  // Load and sanitizes config from file config (default) and merged (user)
  // Takes care of every folder override options.
  function _load_config($debug = false)
  {
    
    $rcmail = rcmail::get_instance();
    
    // Load file configurations into default
    $fpath = $this->home . '/config.inc.php';
    if (is_file($fpath) && is_readable($fpath))
    {
      include($fpath);
      if (is_array($rcmail_config) && $rcmail_config['special_folders'])
      {
        $this->special_folders_defaults = $rcmail_config['special_folders'];
      }
    }

    // Load merged configuration (file + user DB, probably, only user DB) into user
    $this->special_folders_user = $rcmail->config->get('special_folders');
    
    // Merge sanitized configuration
    $this->special_folders = array();
    foreach ($this->special_folders_defaults as $idx => $folder_default)
    {
       // Sanitize folder: may only declare the folder. 
       // If so, sanitize to declare at least folder and folder name
       if(is_string($folder_default))
       {
          $this->special_folders_defaults[$idx] = 
          $folder_default = array(
             'name' => $folder_default,
             'folder' => $folder_default);
       }
       if($this->special_folders_user[$idx] && is_string($this->special_folders_user[$idx]))
       {
          $this->special_folders_user[$idx] = array(
             'name' => $this->special_folders_user[$idx],
             'folder' => $this->special_folders_user[$idx]);
       }
       if(!$this->special_folders_user[$idx])
       {
          $this->special_folders_user[$idx] = array();
       }

       // Create the sanitized folder config
       $this->special_folders[$idx] = array(
          'folder' => 
             ($folder_default['override_folder'] && strlen($this->special_folders_user[$idx]['folder']) ? 
                $this->special_folders_user[$idx]['folder'] : 
                $folder_default['folder']),
          'name'   => 
             ($folder_default['override_name']   && strlen($this->special_folders_user[$idx]['name']) ? 
                $this->special_folders_user[$idx]['name']   : 
                $folder_default['name']),
          'icon'   => 
             ($folder_default['override_icon']   && strlen($this->special_folders_user[$idx]['icon']) ? 
                $this->special_folders_user[$idx]['icon']   : 
                $folder_default['icon']),
          );
    }
    
    if($debug && 0)
    {
       echo "Default:\n";
       print_r($this->special_folders_defaults);
       echo "\nUser:\n";
       print_r($this->special_folders_user);
       echo "\nFinal:\n";
       print_r($this->special_folders);
       
    }
    
    return false;
  }
  
  // Renders 
  function render_mailboxlist($p)
  {
    $rcmail = rcmail::get_instance();

    $p['list'] = $this->_sort_mailbox_list($p['list']);

    // set localized name for the configured special_folders folder
    if ($this->special_folders)
    {
      // set env variable for client
      foreach ($this->special_folders as $idx => $folder)
      {
         if (isset($p['list'][$folder['folder']]))
           $p['list'][$folder['folder']]['name'] = $folder['name'];
         else // search in subfolders
           $this->_mod_folder_name($p['list'], $folder['folder'], $folder['name']);
      }
    }

    return $p;
  }

   /**
   * Sort mailboxes first by default folders and then in alphabethical order
   * @access private
   */
   private function _sort_mailbox_list($a_folders)
   {
      $a_special = $a_defaults = $folders = array();

      // find default folders and skip folders starting with '.'
      foreach ($a_folders as $folder => $content) 
      {
         if ($folder[0] == '.')
            continue;

         if (($p = array_search($folder, $this->special_folders)) !== false && !$a_defaults[$p])
            $a_special[$folder] = $content;
         else if (($p = array_search($folder, $this->default_folders)) !== false && !$a_defaults[$p])
            $a_defaults[$folder] = $content;
         else
            $folders[$folder] = $content;
      }

      // Place default folders on the top, then special ones and finally the rest
      $f = array_merge($a_special, $a_defaults);
      $folders = array_merge($f, $folders);
      
      return $folders;
   }

  function _mod_folder_name(&$list, $folder, $new_name)
  {
    foreach ($list as $idx => $item)
    {
      if ($item['id'] == $folder)
      {
        $list[$idx]['name'] = $new_name;
	return true;
      } 
      else if (!empty($item['folders']))
      {
        if ($this->_mod_folder_name($list[$idx]['folders'], $folder, $new_name))
	  return true;
      }
    }
    return false;
  }

  // Returns a list of files present on $d directory matching $x regexp
  function _file_list($d,$x = false)
  {
    if(is_string($d))
       $d = array($d);
    
    $l = array();
    foreach($d as $dir)
    {
       foreach(array_diff(scandir($dir),array('.','..')) as $f)
       {
          if(is_file($dir . '/' . $f) && 
            (($x) ? ereg($x . '$' , $f) : 1))
          {
             $l[] = $f;
          }
       }
    }
    
    return $l;
  }
             
  function prefs_table($args)
  {
    if ($args['section'] == 'folders')
    {
      $rcmail = rcmail::get_instance();
      $this->add_texts('localization');
      $skin_path = $this->local_skin_path();
      $this->_load_config();
      
      // Create the select for folders
      $select = rcmail_mailbox_select(array('noselection' => '---', 'realnames' => true,
        'maxlength' => 30, 'exceptions' => array('INBOX')));

      // Create the select for icons
      $images = $this->_file_list($this->url($skin_path . '/img'));
      $select_icon = new html_select();
      foreach($images as $image)
      {
         $select_icon->add($image, $image);
      }
      
      // For each special folder defined, show for modification (if overridible): folder, folder name and icon
      foreach ($this->special_folders as $idx => $folder) 
      {
         if($this->special_folders_defaults[$idx]['override_folder'])
         {
            $args['blocks']['main']['options']['special_folders_' . $idx] = array(
                'title' => $this->gettext('special_folders.folder') . ': ' . $folder['name'],
                'content' => $select->show($folder['folder'], array('name' => "_special_folders_" . $idx ))
               );
         }
            
         if($this->special_folders_defaults[$idx]['override_name'])
         {
            $input_name = 
               new html_inputfield(
                  array(
                    'name' => '_special_folders_name_' . $idx,
                    'id'   => '_special_folders_name_' . $idx,
                    'size' => 20));
            $args['blocks']['main']['options']['special_folders_name_' . $idx] = array(
                'title' => $this->gettext('special_folders.name') . ': ' . $folder['name'],
                'content' => $input_name->show($folder['name'])
               );
         }

         if($this->special_folders_defaults[$idx]['override_icon'])
         {
            $img_icon = html::img(array('src' => $this->url($skin_path . "/img/" . $folder['icon']), 'id' => '_special_folders_icon_image_' . $idx));
            $args['blocks']['main']['options']['special_folders_icon_' . $idx] = array(
                'title' => $this->gettext('special_folders.icon') . ': ' . $folder['name'],
                'content' => $select_icon->show($folder['icon'], array(
                  'name' => '_special_folders_icon_' . $idx, 
                  'id' => '_special_folders_icon_' . $idx, 
                  'onchange' => "document.getElementById('_special_folders_icon_image_" . $idx . "').src = '" . $this->url($skin_path) . "/img/' + this.value")
                  ) . $img_icon
               );
         }
      }
    }

    return $args;
  }

  function save_prefs($args)
  {
    if ($args['section'] == 'folders')
    {
      $rcmail = rcmail::get_instance();
      $this->_load_config();

      foreach ($this->special_folders_defaults as $idx => $folder)
      {
         if($folder['override_folder'] == true)
            $this->special_folders[$idx]['folder'] = get_input_value('_special_folders_' . $idx, RCUBE_INPUT_POST);
         if($folder['override_name'])
            $this->special_folders[$idx]['name'] = get_input_value('_special_folders_name_' . $idx, RCUBE_INPUT_POST);
         if($folder['override_icon'])
            $this->special_folders[$idx]['icon'] = get_input_value('_special_folders_icon_' . $idx, RCUBE_INPUT_POST);
      }
      
      $args['prefs']['special_folders'] = $this->special_folders;
      
      return $args;
    }
  }
}
